#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "main.h"
#include "playfield_editor.h"
#include "level_ctrl.h"
#include "image_win_ctrl.h"
#include "level_properties.h"

extern Level *level;
extern Theme *theme;

typedef enum {
	EDIT_UNDEFINED,
	EDIT_LEVEL,
	EDIT_THEME
} GlobalEditMode;

typedef enum {	
	EDIT_LEVEL_UNDEFINED,
	EDIT_LEVEL_PLAYFIELD,
	EDIT_LEVEL_GOAL,
	EDIT_LEVEL_PROPERTIES
} LevelEditMode;

void on_fileselector_ok (GtkWidget *widget, GtkFileSelection *file_selector);
void on_fileselector_cancel (GtkWidget *widget, GtkFileSelection *file_selector);

GlobalEditMode get_global_edit_mode(void);

LevelEditMode get_level_edit_mode(void);


GlobalEditMode get_global_edit_mode(void)
{
	GlobalEditMode result;
	GtkNotebook *level_theme_notebook;
	gint active_page;
	
	level_theme_notebook = GTK_NOTEBOOK(lookup_widget(get_app(), 
							  "level_theme_notebook"));
	active_page = gtk_notebook_current_page(level_theme_notebook);
	switch(active_page)
	{
	case 0: 
		result = EDIT_LEVEL;
		break;
	case 1:
		result = EDIT_THEME;
		break;
	default:
		result = EDIT_UNDEFINED;
		break;
	}	
	return result;
}

LevelEditMode get_level_edit_mode(void)
{
	LevelEditMode result;
	gint active_page;
	GtkNotebook *level_notebook = GTK_NOTEBOOK(lookup_widget(get_app(),
								 "level_notebook"));
	
	/* check whether the level or goal editor is visible */
	active_page = gtk_notebook_current_page(level_notebook);
	switch(active_page)
	{
	case 0:
		result = EDIT_LEVEL_PLAYFIELD;
		break;
	case 1:
		result = EDIT_LEVEL_GOAL;
		break;
	case 2:
		result = EDIT_LEVEL_PROPERTIES;
		break;
	default:
		result = EDIT_LEVEL_UNDEFINED;
		break;
	}
	return result;
}

void
on_mainframe_destroy_event             (GtkObject       *object,
                                        gpointer         user_data)
{
	GtkWidget *level_notebook;
	PlayFieldEditor *pfe;

	/* avoid function call through destroy process */
	level_notebook = lookup_widget(get_app(), "level_notebook");
	gtk_signal_disconnect_by_func(GTK_OBJECT(level_notebook),
				      (GFunc) on_level_switch_page, NULL);

	/* clean up */
	pfe = get_level_editor();
	if(pfe)
	{
		playfield_editor_destroy(pfe);
	}
	pfe = get_goal_editor();
	if(pfe)
	{
		playfield_editor_destroy(pfe);
	}
	theme_image_win_destroy();
	image_win_ctrl_destroy();
	level_ctrl_destroy();
	
	gtk_main_quit();
}



void
on_exit_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *app;
	app = get_app();
	g_return_if_fail(app != NULL);

	/* save levels if neccessary */
	if(level_ctrl_check_for_modifications())
	{
		GtkWidget *dlg;
		gint button_nr;

		dlg = GTK_WIDGET(gnome_message_box_new("Some levels are unsaved. Save this ones?",
						       GNOME_MESSAGE_BOX_QUESTION,
						       GNOME_STOCK_BUTTON_YES,
						       GNOME_STOCK_BUTTON_NO,
						       NULL));
		button_nr = gnome_dialog_run(GNOME_DIALOG(dlg));
		if(button_nr == 0 /* ok */)
		{
			g_print("saving levels ...");
			level_ctrl_save_all_levels();
		}
	}

	gtk_widget_destroy(app);    	
}


void
on_level_clear_activate                (GtkWidget        *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *dlg;
	gint button;
	
	dlg = GTK_WIDGET(gnome_message_box_new(_("Do you really want to clear the grid?"),
					       GNOME_MESSAGE_BOX_QUESTION,
					       GNOME_STOCK_BUTTON_YES, 
					       GNOME_STOCK_BUTTON_NO, NULL));
	button = gnome_dialog_run(GNOME_DIALOG(dlg));

	if(button == 0 /* yes */)
	{
		LevelEditMode mode;
		PlayFieldEditor *pfe = NULL;

		mode = get_level_edit_mode();

		switch(mode)
		{
		case EDIT_LEVEL_PLAYFIELD:
			pfe = get_level_editor();
			break;
		case EDIT_LEVEL_GOAL:
			pfe = get_goal_editor();
			break;
		default:
			break;
		}

		if(pfe != NULL)
		{
			playfield_editor_clear(pfe);
			playfield_clear(pfe->pf);
			level_ctrl_set_actual_level_modified();
		}
	}
}


void
on_preferences_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_level_list_select_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	Level *level;
	PlayFieldEditor *level_editor;
	PlayFieldEditor *goal_editor;

	g_print("select new row\n");

	level = level_ctrl_get_level(row);
	if(level != level_ctrl_get_actual_level())	      
	{
		/* write back properties of previous selected level */
		if(level_properties_changed())
		{
			g_print("properties changed\n");
			level_properties_update_level();

			if(level_properties_name_changed())
			{
				g_print("level renamed\n");
				level_ctrl_actual_level_renamed();
			}
			level_ctrl_set_actual_level_modified();
		}

		/* update all views with new level informations */
		level_ctrl_select_row(row);
		level_editor = get_level_editor();
		goal_editor = get_goal_editor();
		playfield_editor_set_playfield(level_editor, level->playfield);
		playfield_editor_set_playfield(goal_editor, level->goal);
		level_properties_set_level(level);
	}
#ifdef DEBUG
	else
	{
		g_print("Select row: level is the same as before.\n");
	}
#endif

}


void
on_grid_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	PlayFieldEditor *pfe;
	GtkWidget *button;
	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
	gboolean state = item->active;
	
	pfe = get_level_editor();	
	playfield_editor_show_grid(pfe, state);
	pfe =  get_goal_editor();	
	playfield_editor_show_grid(pfe, state);

	button = GTK_WIDGET(lookup_widget(get_app(), "grid_bt"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), state);
	
}

void
on_grid_bt_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *menu_item;
	PlayFieldEditor *pfe;
	gboolean state = gtk_toggle_button_get_active(togglebutton);
	
	pfe = get_level_editor();	
	playfield_editor_show_grid(pfe, state);
	pfe =  get_goal_editor();	
	playfield_editor_show_grid(pfe, state);

	menu_item = GTK_WIDGET(lookup_widget(get_app(), "item_grid"));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), state);
}


void
on_moveable_list_select_icon           (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_moveable(icon);
}


void
on_connection_list_select_icon         (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_connection(icon);
}


void
on_connection_list_unselect_icon       (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_unselect_connection(icon);	
}


void
on_obstacle_list_select_icon           (GnomeIconList   *gnomeiconlist,
                                        gint             icon,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	image_win_ctrl_select_obstacle(icon);
}


void
on_new_file_activate                   (GtkWidget     *item,
                                        gpointer       user_data)
{
	GlobalEditMode mode;

	mode = get_global_edit_mode();
	
	if(mode == EDIT_LEVEL)
	{		
		GtkWidget *new_level_win;
		GtkWidget *level_name;
		GtkWidget *level_file;
		GtkWidget *theme_selector;
		GtkWidget *level_row;
		GtkWidget *level_col;
		GtkWidget *goal_row;
		GtkWidget *goal_col;
		
		gchar *name = NULL;
		gchar *file = NULL;
		gchar *theme = NULL;
		guint pf_row = 10;
		guint pf_col = 10;
		guint gl_row = 10;
		guint gl_col = 10;
				
		gint button;
		gboolean accepted = FALSE;
		gboolean cancel = FALSE;

		new_level_win = create_new_level_win();
		name = g_strdup("noname");
		
		while(!accepted && !cancel)
		{
			/* init level name */
			level_name = lookup_widget(new_level_win, "level_name");
			gtk_entry_set_text(GTK_ENTRY(level_name), name);
			gtk_entry_select_region(GTK_ENTRY(level_name), 0, -1);
			gtk_widget_grab_focus(GTK_WIDGET(level_name));
			g_free(name);
						
			/* init file name */
			level_file = lookup_widget(new_level_win, "level_file");
			gtk_entry_set_text(GTK_ENTRY(level_file), "noname.atomix");
			
			/* init theme selector */
			theme_selector = lookup_widget(new_level_win, "theme_selector");
			gtk_entry_set_text(GTK_ENTRY(theme_selector), "fabian");
			
			/* init level row/col */
			level_row = lookup_widget(new_level_win, "level_row");
			level_col = lookup_widget(new_level_win, "level_col");
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(level_row), 10);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(level_col), 10);
			
			/* init goal row/col */
			goal_row = lookup_widget(new_level_win, "goal_row");
			goal_col = lookup_widget(new_level_win, "goal_col");
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(goal_row), 10);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(goal_col), 10);
			
			button = gnome_dialog_run(GNOME_DIALOG(new_level_win));

			if(button == 0)
			{
				/* get values */
				name = gtk_editable_get_chars(GTK_EDITABLE(level_name),0,-1);
				file = gtk_editable_get_chars(GTK_EDITABLE(level_file),0,-1);
				theme = gtk_editable_get_chars(GTK_EDITABLE(theme_selector),0,-1);
				pf_row = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(level_row));
				pf_col = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(level_col));
				gl_row = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(goal_row));
				gl_col = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(goal_col));
				
			}
			else
			{
				cancel = TRUE;
			}

			if(level_ctrl_does_name_exists(name))
			{
				gtk_entry_select_region(GTK_ENTRY(level_name), 0, -1);
			}
			else
			{
				accepted = TRUE;
			}
		}		

		if(!cancel)
		{
			/* insert into level ctrl */
			level_ctrl_new_level(name, file, theme, pf_row, pf_col,
					     gl_row, gl_col);
		}
		
		gtk_widget_destroy(GTK_WIDGET(new_level_win));
	}
	else if(mode == EDIT_THEME)
	{
		g_print("theme creation not supported yet.\n");
	}
}


void
on_level_resize_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GlobalEditMode mode;

	mode = get_global_edit_mode();

	if(mode == EDIT_LEVEL)
	{
		PlayFieldEditor *pfe = NULL;
		LevelEditMode level_mode;

		level_mode = get_level_edit_mode();
		switch(level_mode)
		{
		case EDIT_LEVEL_PLAYFIELD:
			pfe = get_level_editor();
			break;
		case EDIT_LEVEL_GOAL:
			pfe = get_goal_editor();
			break;
		default:
			break;
		}
		
		if(pfe)
		{
			GtkWidget *window;
			GtkWidget *old_row_entry;
			GtkWidget *old_col_entry;
			GtkWidget *new_row_entry;
			GtkWidget *new_col_entry;
			gint n_rows, n_cols;
			gchar* str_buffer;
			gint button;

			/* create window */
			window = create_resize_grid_win();

			/* get entry widgets */
			old_row_entry = lookup_widget(window, "old_row");
			old_col_entry = lookup_widget(window, "old_column");
			new_row_entry = lookup_widget(window, "new_row");
			new_col_entry = lookup_widget(window, "new_column");
			
			/* set values */
			n_rows = pfe->pf->n_rows;
			n_cols = pfe->pf->n_cols;
			str_buffer = g_malloc(5*sizeof(gchar));
			g_snprintf(str_buffer, 5, "%i", n_rows);
			gtk_entry_set_text(GTK_ENTRY(old_row_entry), str_buffer);
			g_snprintf(str_buffer, 5, "%i", n_cols);
			gtk_entry_set_text(GTK_ENTRY(old_col_entry), str_buffer);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(new_row_entry),
						 n_rows);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(new_col_entry),
						 n_cols);
			g_free(str_buffer);
			
			/* show dialog */
			button = gnome_dialog_run(GNOME_DIALOG(window));

			if(button == 0)
			{
				/* user clicked ok */
				n_rows = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(new_row_entry));
				n_cols = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(new_col_entry));
				playfield_editor_resize(pfe, n_rows, n_cols);
			}
			gtk_widget_destroy(window);
		}
	}
}

void
on_save_activate                       (GtkWidget     *item,
                                        gpointer         user_data)
{
	level_ctrl_save_all_levels();
}


void
on_level_delete_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GlobalEditMode mode;
	mode = get_global_edit_mode();
	
	if(mode == EDIT_LEVEL)
	{
		GtkWidget *dlg;
		gint button;
		
		dlg = GTK_WIDGET(gnome_message_box_new("Do you really want to delete this level?",
						       GNOME_MESSAGE_BOX_QUESTION,
						       GNOME_STOCK_BUTTON_YES, 
						       GNOME_STOCK_BUTTON_NO, NULL));
		button = gnome_dialog_run(GNOME_DIALOG(dlg));
		if(button == 0 /* yes */)
		{
			level_properties_delete_level();
			level_ctrl_remove_actual_level();
		}
	}
}


void
on_level_up_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	level_ctrl_move_up();
}


void
on_level_down_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	level_ctrl_move_down();
}


void
on_level_switch_page                   (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
	if(page_num == 2 /* level properties */)
	{
		level_properties_update_view();
	}
	else
	{
		if(level_properties_changed())
		{
			g_print("properties changed\n");
			level_properties_update_level();

			if(level_properties_name_changed())
			{
				level_ctrl_actual_level_renamed();
			}
			level_ctrl_set_actual_level_modified();
		}

		g_print("other page .. \n");
	}
}

void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget* dlg;
	dlg = create_about_dialog();
	gtk_widget_show(dlg);

}


gboolean
on_mainframe_delete_event              (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	on_exit_activate(0,0);
	
	return FALSE;
}


void
on_theme_window_item_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *button;
	GtkCheckMenuItem *item = GTK_CHECK_MENU_ITEM(menuitem);
	gboolean state = item->active;
	
	if(state)
	{
		theme_image_win_show();
	}
	else
	{
		theme_image_win_hide();
	}

	button = GTK_WIDGET(lookup_widget(get_app(), "theme_wnd_bt"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), state);
	

}


void
on_theme_wnd_bt_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *menu_item;
	gboolean state = gtk_toggle_button_get_active(togglebutton);
	
	if(state)
	{
		theme_image_win_show();
	}
	else
	{
		theme_image_win_hide();
	}

	menu_item = GTK_WIDGET(lookup_widget(get_app(), "theme_window_item"));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), state);

}


gboolean
on_theme_image_win_delete_event        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	GtkWidget *button;
	GtkWidget *menu_item;

	theme_image_win_hide();

	/* update button state */
	menu_item = GTK_WIDGET(lookup_widget(get_app(), "theme_window_item"));
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), FALSE);

	button = GTK_WIDGET(lookup_widget(get_app(), "theme_wnd_bt"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
	
	return TRUE;
}

void on_fileselector_ok(GtkWidget *button, GtkFileSelection *file_selector) 
{
	gchar *selected_filename;

	g_print("yahoo accepted: ");	       
	selected_filename = gtk_file_selection_get_filename(file_selector);
	
	
	if(selected_filename)
	{
		g_print("%s\n", selected_filename);
		level_ctrl_load_level(selected_filename);
	}
	
	gtk_widget_destroy(GTK_WIDGET(file_selector));
}

void on_fileselector_cancel(GtkWidget *button, GtkFileSelection *file_selector) 
{
	gtk_widget_destroy(GTK_WIDGET(file_selector));
}

void
on_open_activate                       (GtkWidget     *item,
                                        gpointer         user_data)
{
	GtkFileSelection *file_selector;

	/* Create file selector */     
	file_selector = GTK_FILE_SELECTION(gtk_file_selection_new("Please select a file for editing."));
	gtk_window_set_modal(GTK_WINDOW(file_selector), TRUE);


	gtk_signal_connect (GTK_OBJECT (file_selector->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (on_fileselector_ok), file_selector);
	gtk_signal_connect (GTK_OBJECT (file_selector->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (on_fileselector_cancel), GTK_OBJECT(file_selector));

	gtk_window_set_position (GTK_WINDOW (file_selector), GTK_WIN_POS_MOUSE);

     
	/* add match pattern */
	gtk_file_selection_complete(GTK_FILE_SELECTION(file_selector),
				    "*.atomix");


	/* Display that dialog */	
	gtk_widget_show (GTK_WIDGET(file_selector));

}

void
on_img_win_ctrl_set_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton))
	{
		image_win_ctrl_set_mouse_function(IMG_WIN_CTRL_SET);
	}
}


void
on_img_win_ctrl_move_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton))
	{
		image_win_ctrl_set_mouse_function(IMG_WIN_CTRL_MOVE);
	}
}


void
on_img_win_ctrl_clear_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton))
	{
		image_win_ctrl_set_mouse_function(IMG_WIN_CTRL_CLEAR);
	}
}

void
on_copy_goal_activate(GtkMenuItem *item, gpointer user_data)
{
	PlayFieldEditor *pfe_goal;
	PlayFieldEditor *pfe_level; 
	PlayField *pf;
	
	GlobalEditMode global_mode = get_global_edit_mode();
	LevelEditMode level_mode = get_level_edit_mode();

	g_return_if_fail(global_mode == EDIT_LEVEL);
	g_return_if_fail(level_mode = EDIT_LEVEL_PLAYFIELD);
	
	pfe_goal = get_goal_editor();
	pfe_level = get_level_editor();

	g_return_if_fail(pfe_goal != NULL);
	g_return_if_fail(pfe_level != NULL);       
	
	pf = playfield_strip(pfe_goal->pf);
	playfield_editor_insert_pf(pfe_level, pf);
}
